Core Exceptions — SQLAlchemy 1.4 Documentation

Release: 1.4.43 | Release Date: November 4, 2022

# SQLAlchemy 1.4 Documentation

# Core Exceptions

Exceptions used with SQLAlchemy.

The base exception class is `SQLAlchemyError`. Exceptions which are raised as a result of DBAPI exceptions are all subclasses of `DBAPIError`.

exception sqlalchemy.exc.AmbiguousForeignKeysError(*arg, **kw)

  Raised when more than one foreign key matching can be located between two selectables during a join.

exception sqlalchemy.exc.ArgumentError(*arg, **kw)

  Raised when an invalid or conflicting function argument is supplied.

  This error generally corresponds to construction time state errors.

exception sqlalchemy.exc.AwaitRequired(*arg, **kw)

  Error raised by the async greenlet spawn if no async operation was awaited when it required one.

exception sqlalchemy.exc.Base20DeprecationWarning(*arg, **kw)

  Issued for usage of APIs specifically deprecated or legacy in SQLAlchemy 2.0.

  See also

  The <some function> in SQLAlchemy 2.0 will no longer <something>.

  SQLAlchemy 2.0 Deprecations Mode

  attribute `sqlalchemy.exc.Base20DeprecationWarning.` deprecated_since = '1.4'

    Indicates the version that started raising this deprecation warning

exception sqlalchemy.exc.CircularDependencyError(message, cycles, edges, msg=None, code=None)

  Raised by topological sorts when a circular dependency is detected.

  There are two scenarios where this error occurs:

  - In a Session flush operation, if two objects are mutually dependent on each other, they can not be inserted or deleted via INSERT or DELETE statements alone; an UPDATE will be needed to post-associate or pre-deassociate one of the foreign key constrained values. The `post_update` flag described at Rows that point to themselves / Mutually Dependent Rows can resolve this cycle.

  - In a `MetaData.sorted_tables` operation, two `ForeignKey` or `ForeignKeyConstraint` objects mutually refer to each other. Apply the `use_alter=True` flag to one or both, see Creating/Dropping Foreign Key Constraints via ALTER.

exception sqlalchemy.exc.CompileError(*arg, **kw)

  Raised when an error occurs during SQL compilation

exception sqlalchemy.exc.DBAPIError(statement, params, orig, hide_parameters=False, connection_invalidated=False, code=None, ismulti=None)

  Raised when the execution of a database operation fails.

  Wraps exceptions raised by the DB-API underlying the database operation. Driver-specific implementations of the standard DB-API exception types are wrapped by matching sub-types of SQLAlchemy’s `DBAPIError` when possible. DB-API’s `Error` type maps to `DBAPIError` in SQLAlchemy, otherwise the names are identical. Note that there is no guarantee that different DB-API implementations will raise the same exception type for any given error condition.

  `DBAPIError` features `StatementError.statement` and `StatementError.params` attributes which supply context regarding the specifics of the statement which had an issue, for the typical case when the error was raised within the context of emitting a SQL statement.

  The wrapped exception object is available in the `StatementError.orig` attribute. Its type and properties are DB-API implementation specific.

exception sqlalchemy.exc.DataError(statement, params, orig, hide_parameters=False, connection_invalidated=False, code=None, ismulti=None)

  Wraps a DB-API DataError.

exception sqlalchemy.exc.DatabaseError(statement, params, orig, hide_parameters=False, connection_invalidated=False, code=None, ismulti=None)

  Wraps a DB-API DatabaseError.

exception sqlalchemy.exc.DisconnectionError(*arg, **kw)

  A disconnect is detected on a raw DB-API connection.

  This error is raised and consumed internally by a connection pool. It can be raised by the `PoolEvents.checkout()` event so that the host pool forces a retry; the exception will be caught three times in a row before the pool gives up and raises `InvalidRequestError` regarding the connection attempt.

Object Name | Description
--- | ---
DontWrapMixin | A mixin class which, when applied to a user-defined Exception class, will not be wrapped inside of `StatementError` if the error is emitted within the process of executing a statement.
HasDescriptionCode | helper which adds ‘code’ as an attribute and ‘_code_str’ as a method

class sqlalchemy.exc.DontWrapMixin

  A mixin class which, when applied to a user-defined Exception class, will not be wrapped inside of `StatementError` if the error is emitted within the process of executing a statement.

  E.g.:

      from sqlalchemy.exc import DontWrapMixin

      class MyCustomException(Exception, DontWrapMixin):
          pass

      class MySpecialType(TypeDecorator):
          impl = String

          def process_bind_param(self, value, dialect):
              if value == 'invalid':
                  raise MyCustomException("invalid!")

class sqlalchemy.exc.HasDescriptionCode(*arg, **kw)

  helper which adds ‘code’ as an attribute and ‘_code_str’ as a method

exception sqlalchemy.exc.IdentifierError(*arg, **kw)

  Raised when a schema name is beyond the max character limit

exception sqlalchemy.exc.IntegrityError(statement, params, orig, hide_parameters=False, connection_invalidated=False, code=None, ismulti=None)

  Wraps a DB-API IntegrityError.

exception sqlalchemy.exc.InterfaceError(statement, params, orig, hide_parameters=False, connection_invalidated=False, code=None, ismulti=None)

  Wraps a DB-API InterfaceError.

exception sqlalchemy.exc.InternalError(statement, params, orig, hide_parameters=False, connection_invalidated=False, code=None, ismulti=None)

  Wraps a DB-API InternalError.

exception sqlalchemy.exc.InvalidRequestError(*arg, **kw)

  SQLAlchemy was asked to do something it can’t do.

  This error generally corresponds to runtime state errors.

exception sqlalchemy.exc.InvalidatePoolError(*arg, **kw)

  Raised when the connection pool should invalidate all stale connections.

  A subclass of `DisconnectionError` that indicates that the disconnect situation encountered on the connection probably means the entire pool should be invalidated, as the database has been restarted.

  This exception will be handled otherwise the same way as `DisconnectionError`, allowing three attempts to reconnect before giving up.

  New in version 1.2.

exception sqlalchemy.exc.LegacyAPIWarning(*arg, **kw)

  indicates an API that is in ‘legacy’ status, a long term deprecation.

exception sqlalchemy.exc.MissingGreenlet(*arg, **kw)

  Error raised by the async greenlet await_ if called while not inside the greenlet spawn context.

exception sqlalchemy.exc.MovedIn20Warning(*arg, **kw)

  Subtype of RemovedIn20Warning to indicate an API that moved only.

exception sqlalchemy.exc.MultipleResultsFound(*arg, **kw)

  A single database result was required but more than one were found.

  Changed in version 1.4: This exception is now part of the `sqlalchemy.exc` module in Core, moved from the ORM. The symbol remains importable from `sqlalchemy.orm.exc`.

exception sqlalchemy.exc.NoForeignKeysError(*arg, **kw)

  Raised when no foreign keys can be located between two selectables during a join.

exception sqlalchemy.exc.NoInspectionAvailable(*arg, **kw)

  A subject passed to `sqlalchemy.inspection.inspect()` produced no context for inspection.

exception sqlalchemy.exc.NoReferenceError(*arg, **kw)

  Raised by `ForeignKey` to indicate a reference cannot be resolved.

exception sqlalchemy.exc.NoReferencedColumnError(message, tname, cname)

  Raised by `ForeignKey` when the referred `Column` cannot be located.

exception sqlalchemy.exc.NoReferencedTableError(message, tname)

  Raised by `ForeignKey` when the referred `Table` cannot be located.

exception sqlalchemy.exc.NoResultFound(*arg, **kw)

  A database result was required but none was found.

  Changed in version 1.4: This exception is now part of the `sqlalchemy.exc` module in Core, moved from the ORM. The symbol remains importable from `sqlalchemy.orm.exc`.

exception sqlalchemy.exc.NoSuchColumnError(*arg, **kw)

  A nonexistent column is requested from a `Row`.

exception sqlalchemy.exc.NoSuchModuleError(*arg, **kw)

  Raised when a dynamically-loaded module (usually a database dialect) of a particular name cannot be located.

exception sqlalchemy.exc.NoSuchTableError(*arg, **kw)

  Table does not exist or is not visible to a connection.

exception sqlalchemy.exc.NotSupportedError(statement, params, orig, hide_parameters=False, connection_invalidated=False, code=None, ismulti=None)

  Wraps a DB-API NotSupportedError.

exception sqlalchemy.exc.ObjectNotExecutableError(target)

  Raised when an object is passed to .execute() that can’t be executed as SQL.

  New in version 1.1.

exception sqlalchemy.exc.OperationalError(statement, params, orig, hide_parameters=False, connection_invalidated=False, code=None, ismulti=None)

  Wraps a DB-API OperationalError.

exception sqlalchemy.exc.PendingRollbackError(*arg, **kw)

  A transaction has failed and needs to be rolled back before continuing.

  New in version 1.4.

exception sqlalchemy.exc.ProgrammingError(statement, params, orig, hide_parameters=False, connection_invalidated=False, code=None, ismulti=None)

  Wraps a DB-API ProgrammingError.

exception sqlalchemy.exc.RemovedIn20Warning(*arg, **kw)

  indicates an API that will be fully removed in SQLAlchemy 2.0.

exception sqlalchemy.exc.ResourceClosedError(*arg, **kw)

  An operation was requested from a connection, cursor, or other object that’s in a closed state.

exception sqlalchemy.exc.SADeprecationWarning(*arg, **kw)

  Issued for usage of deprecated APIs.

  attribute `sqlalchemy.exc.SADeprecationWarning.` deprecated_since = None

    Indicates the version that started raising this deprecation warning

exception sqlalchemy.exc.SAPendingDeprecationWarning

  A similar warning as `SADeprecationWarning`, this warning is not used in modern versions of SQLAlchemy.

  attribute `sqlalchemy.exc.SAPendingDeprecationWarning.` deprecated_since = None

    Indicates the version that started raising this deprecation warning

exception sqlalchemy.exc.SAWarning(*arg, **kw)

  Issued at runtime.

exception sqlalchemy.exc.SQLAlchemyError(*arg, **kw)

  Generic error class.

exception sqlalchemy.exc.StatementError(message, statement, params, orig, hide_parameters=False, code=None, ismulti=None)

  An error occurred during execution of a SQL statement.

  `StatementError` wraps the exception raised during execution, and features `statement` and `params` attributes which supply context regarding the specifics of the statement which had an issue.

  The wrapped exception object is available in the `orig` attribute.

  attribute `sqlalchemy.exc.StatementError.` orig = None

    The DBAPI exception object.

  attribute `sqlalchemy.exc.StatementError.` params = None

    The parameter list being used when this exception occurred.

  attribute `sqlalchemy.exc.StatementError.` statement = None

    The string SQL statement being invoked when this exception occurred.

exception sqlalchemy.exc.TimeoutError(*arg, **kw)

  Raised when a connection pool times out on getting a connection.

exception sqlalchemy.exc.UnboundExecutionError(*arg, **kw)

  SQL was attempted without a database connection to execute it on.

exception sqlalchemy.exc.UnreflectableTableError(*arg, **kw)

  Table exists but can’t be reflected for some reason.

  New in version 1.2.

exception sqlalchemy.exc.UnsupportedCompilationError(compiler, element_type, message=None)

  Raised when an operation is not supported by the given compiler.

  See also

  How do I render SQL expressions as strings, possibly with bound parameters inlined?

  Compiler StrSQLCompiler can’t render element of type <element type>